library EPG_ISO6973;

uses
  SysUtils,
  Classes,
  Windows,
  Forms,
  inifiles,
  menus,
  ModuleAPI in '..\Include\ModuleAPI.pas',
  Helpers in '..\Include\Helpers.pas',
  main in 'main.pas' {fmain},
  iso6973 in 'iso6973.pas';

{$E mod}

{$R *.res}
//------------------------------------------------------------------------------

function OnGetModuleInfo(ModuleInfo: PModuleInfo): BOOL; stdcall; export;
begin
  ModuleInfo^.dwModuleAPI := MODULE_API_1_5;
  ModuleInfo^.szModuleName := 'EPG ISO 6973';
  ModuleInfo^.szModuleAuthor := 'petrusek';
  ModuleInfo^.szModuleEmail := '';
  ModuleInfo^.dwModuleVer := $01000000; {v1.0.0.0}
  ModuleInfo^.wShorCut := 0;
  ModuleInfo^.dwModuleType := MODTYPE_EPG;
  ModuleInfo^.dwRequiredDDver := $00090009;  //requires DD v0.9.8
  result := false;
end;
//------------------------------------------------------------------------------

function OnInit(pInitInfo: PInitInfo): BOOL; stdcall; export;

begin
  hwndDD := pInitInfo^.hMainWnd;
  charset := pInitInfo^.CharSet;
  InitHelpers(hwndDD);

  AppPath := ExtractFilePath(Application.ExeName);

  fmain := Tfmain.Create(Application);
  fmain.Font.Charset := charset;
  fmain.LV.Font.Charset := charset;
  fmain.TV.Font.Charset := charset;
  fmain.MDescr.Font.Charset := charset;
  fmain.GroupBox2.Font.Charset := charset;
  fmain.BAdd.Font.Charset := charset;
  fmain.BClear.Font.Charset := charset;
  fmain.BRefresh.Font.Charset := charset;
  result := false;
end;
//------------------------------------------------------------------------------

function OnClose(): BOOL; stdcall; export;
begin
  fmain.Free;
  DeleteCriticalSection(LockCS);
  result := false;
end;
//------------------------------------------------------------------------------

function OnShow(): BOOL; stdcall; export;
begin
  InitializeCriticalSection(LockCS);

  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_CHL_GET_POINTER, Integer(@chl));
  fmain.Show;

  result := false;
end;
//------------------------------------------------------------------------------

function OnDDEvent(EventType: DWORD; EventParams: Pointer): BOOL; stdcall;
  export;
var
  dwMode: DWORD;
  chid: Integer;
  e: PEPGInfo;
begin
  result := false;
  case EventType of
    DDEVENT_ON_REMOTE: begin
        if not Assigned(fmain) then
          exit;
        case PDWORD(EventParams)^ of
          RMT_EPG: begin
              if fmain.Visible then
                fmain.Close
              else begin
                SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_GET_DISPLAY_MODE,
                  Integer(@dwMode));
                if dwMode = DISPLAYMODE_FULLSCREEN then
                  fmain.WindowState := wsMaximized
                else
                  fmain.WindowState := wsNormal;
                fmain.Show;
                fmain.BRefresh.Click;
                SetWindowPos(fmain.Handle, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOSIZE
                  or SWP_NOMOVE);
              end;
            end;

          RMT_UP: begin
              if not fmain.Visible then
                exit;
              if fmain.TV.Focused then begin
                if fmain.TV.Selected = nil then
                  if fmain.TV.Items.Count > 0 then
                    fmain.TV.Items[0].Selected := true;

                if fmain.TV.Items.Count > 0 then begin
                  if fmain.TV.Selected.Index > 0 then begin
                    fmain.TV.Items[fmain.TV.Selected.Index - 1].Selected :=
                      true;
                  end;
                end;
              end
              else begin
                if fmain.LV.Selected = nil then
                  if fmain.LV.Items.Count > 0 then
                    fmain.LV.Items[0].Selected := true;

                if fmain.LV.Items.Count > 0 then begin
                  if fmain.LV.Selected.Index > 0 then begin
                    fmain.LV.Items[fmain.LV.Selected.Index - 1].Selected :=
                      true;
                  end;
                end;
              end;
            end;

          RMT_DOWN: begin
              if not fmain.Visible then
                exit;

              if fmain.TV.Focused then begin
                if fmain.TV.Selected = nil then
                  if fmain.TV.Items.Count > 0 then
                    fmain.TV.Items[0].Selected := true;

                if fmain.TV.Items.Count > 0 then begin
                  if fmain.TV.Selected.Index < (fmain.TV.Items.Count - 1) then
                    begin
                    fmain.TV.Items[fmain.TV.Selected.Index + 1].Selected :=
                      true;
                  end;
                end;
              end
              else begin
                if fmain.LV.Selected = nil then
                  if fmain.LV.Items.Count > 0 then
                    fmain.LV.Items[0].Selected := true;

                if fmain.LV.Items.Count > 0 then begin
                  if fmain.LV.Selected.Index < (fmain.LV.Items.Count - 1) then
                    begin
                    fmain.LV.Items[fmain.LV.Selected.Index + 1].Selected :=
                      true;
                  end;
                end;
              end;
            end;

          RMT_LEFT: begin
              if not fmain.Visible then
                exit;

              fmain.TV.SetFocus;
              if fmain.TV.Selected = nil then
                if fmain.TV.Items.Count > 0 then
                  fmain.TV.Items[0].Selected := true;
            end;

          RMT_RIGHT: begin
              if not fmain.Visible then
                exit;

              fmain.LV.SetFocus;
              if fmain.LV.Selected = nil then
                if fmain.LV.Items.Count > 0 then
                  fmain.LV.Items[0].Selected := true;
            end;

          RMT_OK: begin
              if fmain.TV.Selected = nil then exit;
              if fmain.TV.Selected.Data = nil then exit;
              e :=fmain.TV.Selected.Data;
              chid := FindChId(e^.wSat, e^.wNetID, e^.wSID);
              if chid >= 0 then
                SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_SET_CHANNEL_BY_CHID,
                chid);
            end;

           RMT_RECORD: begin
               fmain.BAdd.Click;
           end;

           RMT_MENU: begin
              fmain.BRefresh.Click;
           end;

           RMT_STOP: begin
              fmain.BClear.Click;
           end;

        end;
        if fmain.Visible then
          PDWORD(EventParams)^ := 0;
      end;
  end;
end;
//------------------------------------------------------------------------------

exports OnGetModuleInfo, OnInit, OnClose, OnShow, OnDDEvent;

begin
end.

