// references about ISO 6937
// http://en.wikipedia.org/wiki/ISO/IEC_6937
// http://tv-plattform.de/download/DVB-T/MinAnfo/A1_E-Book2-02.pdf
// http://www.digitalinfo.cz/documents/kodovani_DVB_Hermes.pdf

unit iso6973;

interface

function DecodeAsIso6973(buffer : array of char) : WideString;
function GetSpecialChar(accent : byte; character : byte) : WideChar;

const chcount = 156;

const src : array[1..chcount] of word = (
  // Grave + AEIOUaeiou
  $C141, $C145, $C149, $C14F, $C155, $C161, $C165, $C169, $C16F, $C175,
  // Acute + ACEILNORSUYZaceilnorsuyz
  $C241, $C243, $C245, $C249, $C24C, $C24E, $C24F, $C252, $C253, $C255, $C259, $C25A, $C261, $C263, $C265, $C269, $C26C, $C26E, $C26F, $C272, $C273, $C275, $C279, $C27A,
  // Circumflex + ACEGHIJOSUWYaceghijosuwy
  $C341, $C343, $C345, $C347, $C348, $C349, $C34A, $C34F, $C353, $C355, $C357, $C359, $C361, $C363, $C365, $C367, $C368, $C369, $C36A, $C36F, $C373, $C375, $C377, $C379,
  // Tilde + AINOUainou
  $C441, $C449, $C44E, $C44F, $C455, $C461, $C469, $C46E, $C46F, $C475,
  // Macron + AEIOUaeiou
  $C541, $C545, $C549, $C54F, $C555, $C561, $C565, $C569, $C56F, $C575,
  // Breve + AGUagu
  $C641, $C647, $C655, $C661, $C667, $C675,
  // Dot + CEGIZcegiz
  $C743, $C745, $C747, $C749, $C75A, $C763, $C765, $C767, $C769, $C77A,
  // Umlaut + AEIOUYaeiouy
  $C841, $C845, $C849, $C84F, $C855, $C859, $C861, $C865, $C869, $C86F, $C875, $C879,
  // Ring + AUau
  $CA41, $CA55, $CA61, $CA75,
  // Cedilla + CGKLNRSTcgklnrst
  $CB43, $CB47, $CB4B, $CB4C, $CB4E, $CB52, $CB53, $CB54, $CB63, $CB67, $CB6B, $CB6C, $CB6E, $CB72, $CB73, $CB74,
  // DoubleAcute + OUou
  $CD4F, $CD55, $CD6F, $CD75,
  // Ogonek + AEIUaeiu
  $CE41, $CE45, $CE49, $CE55, $CE61, $CE65, $CE69, $CE75,
  // Caron + CDELNRSTZcdelnrstz
  $CF43, $CF44, $CF45, $CF4C, $CF4E, $CF52, $CF53, $CF54, $CF5A, $CF63, $CF64, $CF65, $CF6C, $CF6E, $CF72, $CF73, $CF74, $CF7A
);

const dst : array[1..chcount] of word = (
  // Graved AEIOUaeiou
  $00C0, $00C8, $00CC, $00D2, $00D9, $00E0, $00E8, $00EC, $00F2, $00F9,
  // Acuted ACEILNORSUYZaceilnorsuyz
  $00C1, $0106, $00C9, $00CD, $0139, $0143, $00D3, $0154, $015A, $00DA, $00DD, $0179, $00E1, $0107, $00E9, $00ED, $013A, $0144, $00F3, $0155, $015B, $00FA, $00FD, $017A,
  // Circumflexed ACEGHIJOSUWYaceghijosuwy
  $00C2, $0108, $00CA, $011C, $0124, $00CE, $0134, $00D4, $015C, $00DB, $0174, $0176, $00E2, $0109, $00EA, $011D, $0125, $00EE, $0135, $00F4, $015D, $00FB, $0175, $0177,
  // Tilded AINOUainou
  $00C3, $0128, $00D1, $00D5, $0168, $00E3, $0129, $00F1, $00F5, $0169,
  // Macroned AEIOUaeiou
  $0100, $0112, $012A, $014C, $016A, $0101, $0113, $012B, $014D, $016B,
  // Breved AGUagu
  $0102, $011E, $016C, $0103, $011F, $016D,
  // Dotted CEGIZcegiz
  $010A, $0116, $0120, $0130, $017B, $010B, $0117, $0121, $0131, $017C,
  // Umlautted AEIOUYaeiouy
  $00C4, $00CB, $00CF, $00D6, $00DC, $0178, $00E4, $00EB, $00EF, $00F6, $00FC, $00FF,
  // Ringed AUau
  $00C5, $016E, $00E5, $016F,
  // Cedilled CGKLNRSTcgklnrst
  $00C7, $0122, $0136, $013B, $0145, $0156, $015E, $0162, $00E7, $0123, $0137, $013C, $0146, $0157, $015F, $0163,
  // DoubleAcuted OUou
  $0150, $0170, $0151, $0171,
  // Ogonked :) AEIUaeiu
  $0104, $0118, $012E, $0172, $0105, $0119, $012F, $0173,
  // Caroned CDELNRSTZcdelnrstz
  $010C, $010E, $011A, $013D, $0147, $0158, $0160, $0164, $017D, $010D, $010F, $011B, $013E, $0148, $0159, $0161, $0165, $017E
);

implementation

function DecodeAsIso6973(buffer : array of char): WideString;
var index:integer;
    maxindex:integer;
    ch:word;
begin
  Result := '';
  index := Low(buffer);
  maxindex := High(buffer);
  while (index < maxindex) do begin
    ch := ord(buffer[index]);
    case ch of
      //$24: Result := Result + ''; // $ ->  ???
      $C1..$C8,$CA,$CB,$CD..$CF:
      if (index + 1 < maxindex) then begin
        inc(index);
        Result := Result + GetSpecialChar(ch, ord(buffer[index]));
      end;
    else
      Result := Result + WideChar(ch);
    end;
    inc(index);
  end;
end;

function GetSpecialChar(accent : byte; character : byte) : WideChar;
var i:integer;
    w:word;
begin
  Result := WideChar(character);
  w := (accent shl 8) or character;
  for i := 1 to chcount do begin
    if (src[i] = w) then begin
      Result := WideChar(dst[i]);
      break;
    end;
  end;
end;

end.
