unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, TeEngine, Series, ExtCtrls, TeeProcs, Chart, Grids,
  StdCtrls, Buttons, moduleapi, helpers, dateutils, Menus, iso6973;

const
  MAX_EPG_CHANNELS = 2560;
  MAX_EPG_EVENTS = 256;

type
  PEPGCh = ^TEPGCh;
  TEPGCh = record
    Sat: WORD;
    NID: WORD;
    SID: WORD;
    Name: array[0..255] of char;
    bNotInChbase: boolean;
    Events: array[0..MAX_EPG_EVENTS - 1] of PEPGInfo;
    EventsCount: Integer;
  end;

  TEPGChArray = array[0..MAX_EPG_CHANNELS - 1] of TEPGCh;

var
  EPGChs: ^TEPGChArray;
  EPGChsCount: Integer = 0;
  bEventAlreadyCaptured: boolean;
  bChAlreadyCaptured: boolean;

type

  Tfmain = class(TForm)
    GroupBox2: TGroupBox;
    SB: TStatusBar;
    Panel1: TPanel;
    TV: TTreeView;
    Splitter1: TSplitter;
    Panel2: TPanel;
    LV: TListView;
    Splitter2: TSplitter;
    Panel3: TPanel;
    MDescr: TMemo;
    Panel4: TPanel;
    BRefresh: TBitBtn;
    BAdd: TBitBtn;
    BClear: TBitBtn;
    PopupMenu1: TPopupMenu;
    Switchtochannel1: TMenuItem;
    procedure BRefreshClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TVChange(Sender: TObject; Node: TTreeNode);
    procedure LVChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure LVCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: Integer; var Compare: Integer);
    procedure TVCompare(Sender: TObject; Node1, Node2: TTreeNode;
      Data: Integer; var Compare: Integer);
    procedure BAddClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Switchtochannel1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BClearClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    procedure Reset;
    { Private declarations }
  public
    { Public declarations }
    procedure AddToListView(e: PEpgInfo);
  end;

var
  fmain: Tfmain;
  hwndDD: HWND;
  charset: TFontCharset;
  AppPath: string;
  TotalEPG: Integer;
  chl: PCHL;

  LockCS: TRTLCriticalSection;

function FindChID(wSatPos, wNID, wSID: WORD): Integer;

implementation

{$R *.dfm}

function FindChID(wSatPos, wNID, wSID: WORD): Integer;
var
  i: Integer;
begin
  result := -1;
  for i := 0 to MAX_CHANNELS - 1 do
    if (chl^.Channels[i].dwFreq <> 0) and
       (chl^.Channels[i].wSatPos = wSatPos) and
       (chl^.Channels[i].wNID = wNID) and
       (chl^.Channels[i].wSID = wSID) then
       begin
        result := i;
        break;
       end;
end;

procedure Tfmain.AddToListView(e: PEpgInfo);
var
  li: TListItem;
  s: string;
  s2: string;
begin
  Assert(e <> nil);

  li := LV.Items.Add;
//  datetimetostring(s, 'dd mmm hh:nn', DVBTimeToLocalTime(e^.dwStartTime));
  datetimetostring(s, 'dd mmm hh:nn', UnixToDateTime(DVBTimeToLocalTime(e^.dwStartTime)));
  li.Caption := s;
  datetimetostring(s, 'hh:nn', UnixToDateTime(e^.dwDuration));
  li.SubItems.Add(s);
//  li.SubItems.Add(IntToStr(e^.dwDuration div 60) + ' min');
  s := DecodeAsIso6973(e^.EventName);
  if length(s) > 0 then
    if ord(s[1]) < $20 then delete(s, 1, 1);
  li.SubItems.Add(s);


  s := DecodeAsIso6973(e^.ShortDescr);
  if length(s) > 0 then
    if ord(s[1]) < $20 then delete(s, 1, 1);

  s2 := DecodeAsIso6973(e^.ExtendedDescr);
  if length(s2) > 0 then
    if ord(s2[1]) < $20 then delete(s2, 1, 1);

  li.SubItems.Add(s + ' ' + s2);
  li.Data := e;
end;

var
  CurrentSat: Integer;
  CurrentNID: Integer;
  CurrentSID: Integer;
  EPGCounter: DWORD = 0;


function EPGCallback(pepg: PEPGInfo): Integer; stdcall;
var
  e: TEPGInfo;
  a, b: Integer;
  EPGChsIndex: integer;
//  i: integer;
//  CurrentIndex: Integer;
  n: TTreeNode;
//  str: string;
//  pstr: pchar;

  chid: Integer;
begin
  result := 0;
  EPGChsIndex := 0;
  EnterCriticalSection(LockCS);
  try
  e := pepg^;

  case EPGCounter mod 2 of
    0: fmain.SB.Panels[3].Text := ' ';
    1: fmain.SB.Panels[3].Text := '*';
  end;
  inc(EPGCounter);

  bChAlreadyCaptured := false;
  bEventAlreadyCaptured := false;
  for a := 0 to EPGChsCount - 1 do
    if (EPGChs[a].Sat = e.wSat) and (EPGChs[a].NID = e.wNetID) and
      (EPGChs[a].SID = e.wSID) then begin
      bChAlreadyCaptured := true;
      EPGChsIndex := a;
      for b := 0 to EPGChs[a].EventsCount - 1 do
        if (EPGChs[a].Events[b]^.dwStartTime = e.dwStartTime) then begin
          bEventAlreadyCaptured := true;
          break;
        end;

      break;
    end;


  if not bChAlreadyCaptured then begin
    EPGChs[EPGChsCount].Sat := e.wSat;
    EPGChs[EPGChsCount].NID := e.wNetID;
    EPGChs[EPGChsCount].SID := e.wSID;
    chid := FindChId(e.wSat, e.wNetID, e.wSID);

    if chid >= 0 then begin
      strlcopy(@EPGChs[EPGChsCount].Name, pchar(@chl^.Channels[chid].ChName), 256)
    end
    else
      EPGChs[EPGChsCount].bNotInChbase := true;


    if not EPGChs[EPGChsCount].bNotInChbase then begin
      n := fmain.TV.Items.Add(nil, string(pchar(@EPGChs[EPGChsCount].Name)));
      n.Data := Pointer(EPGChsCount);
      EPGChsIndex := EPGChsCount;
      inc(EPGChsCount);
    end;

    fmain.SB.Panels.Items[0].Text := IntToStr(EPGChsCount) + ' ' + Translate('EPG_CHANNELS_SHORT');
  end;

//  fmain.Memo1.Lines.Add(format('wSat:%d  wNID:%d  wSID:%d',[e.wSat, e.wNetID, e.wSID]));


  if (not bEventAlreadyCaptured) and (not EPGChs[EPGChsIndex].bNotInChbase)
    and (EPGChs[EPGChsIndex].EventsCount < MAX_EPG_EVENTS - 1)
    then begin
    a := EPGChs[EPGChsIndex].EventsCount;
    try
      New(EPGChs[EPGChsIndex].Events[a]);
    except
      Application.MessageBox('can`t allocate memory-space for EPG-event.', 'Error', MB_OK or MB_ICONERROR);
      exit;
    end;
    copymemory(EPGChs[EPGChsIndex].Events[a], @e, sizeof(TEPGInfo));

    if (e.wSat = CurrentSat) and (e.wNetID = CurrentNID) and (e.wSID =
      CurrentSID) then begin
//      CurrentIndex := EPGChsIndex;
      fmain.AddToListView(EPGChs[EPGChsIndex].Events[a]);
    end;

    inc(TotalEPG);
    fmain.SB.Panels.Items[1].Text := inttostr(TotalEPG) + ' ' + Translate('EPG_EVENTS');
    application.ProcessMessages;

    inc(EPGChs[EPGChsIndex].EventsCount);
  end;

  finally
    LeaveCriticalSection(LockCS);
  end;
end;

procedure Tfmain.BRefreshClick(Sender: TObject);
{
var
  e: TEPGInfo;
  a, b: Integer;
  n: TTreeNode;
  str: string;
  pstr: pchar;
  EPGChsIndex: integer;
  i: integer;
  CurrentIndex: Integer;
  }
begin

  BRefresh.Enabled := false;
  application.ProcessMessages;

  fmain.LV.Clear;
//  CurrentIndex := 0;

  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_START_EPG, Integer(@EPGCallback));
end;

procedure Tfmain.Reset;
var
  a: integer;
begin
  try
    GetMem(EPGChs, sizeof(TEPGCh) * MAX_EPG_CHANNELS);
  except
    Application.MessageBox('can`t allocate memory-space for EPG-channels array.', 'Error', MB_OK or MB_ICONERROR);
    exit;
  end;

  for a := 0 to MAX_EPG_CHANNELS - 1 do begin
    EPGChs[a].bNotInChbase := false;
    EPGChs[a].EventsCount := 0;
  end;
  EPGChsCount := 0;
  TotalEPG := 0;
end;

procedure Tfmain.FormCreate(Sender: TObject);
begin
  Reset;
end;

procedure Tfmain.TVChange(Sender: TObject; Node: TTreeNode);
var
  i: integer;
  a: integer;
begin
  //  caption := ':'+TV.Selected.Text;    exit;

  LV.Clear;
  if TV.Selected = nil then exit;
  if TV.SelectionCount <= 0 then    exit;

  if EPGChs = nil then exit;


  a := Integer(TV.Selected.Data);

  if a >= MAX_EPG_CHANNELS then exit;


  GroupBox2.Caption := Translate('ELECTRONIC_PROGRAM_GUIDE');

//  BSwitchToCh.Caption := EPGChs[a].Name;
  GroupBox2.Caption := Translate('ELECTRONIC_PROGRAM_GUIDE') + ' - [' + EPGChs[a].Name + ']';
//  BSwitchToCh.Visible := true;

  for i := 0 to EPGChs[a].EventsCount - 1 do
    AddToListView(EPGChs[a].Events[i]);
end;

procedure Tfmain.LVChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  if item.SubItems.Count > 0 then
    MDescr.Text := trim(item.SubItems.Strings[2]);
end;

procedure Tfmain.LVCompare(Sender: TObject; Item1, Item2: TListItem;
  Data: Integer; var Compare: Integer);
var
  e1, e2: PEPGInfo;
begin
  e1 := Item1.Data;
  e2 := Item2.Data;
  if e1.dwStartTime < e2.dwStartTime then
    Compare := -1;
  if e1.dwStartTime = e2.dwStartTime then
    Compare := 0;
  if e1.dwStartTime > e2.dwStartTime then
    Compare := +1;
end;

procedure Tfmain.TVCompare(Sender: TObject; Node1, Node2: TTreeNode;
  Data: Integer; var Compare: Integer);
var
  a, b: integer;
begin
  a := Integer(Node1.Data);
  b := Integer(Node2.Data);


  Compare := strcomp(@EPGChs[a].Name, EPGChs[b].Name);
end;

procedure Tfmain.BAddClick(Sender: TObject);
var
  e: PEPGInfo;
  sr: TScheduledRecord;
  chid: Integer;
begin
  if LV.SelCount <= 0 then exit;
  e := LV.Selected.Data;
  chid := FindChId(e^.wSat, e^.wNetID, e^.wSID);

  if chid >= 0 then begin
//    addlog('[DDEpg] e^.dwStartTime=' + inttostr(e^.dwStartTime));
    sr.dwSize := sizeof(TScheduledRecord);
    sr.dwStartTime := DVBTimeToLocalTime(e^.dwStartTime);
    sr.dwStopTime := DVBTimeToLocalTime(e^.dwStartTime + e^.dwDuration);
    sr.dwChID := chid;
    SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_ADD_SCHEDULED_RECORD, Integer(@sr));
{    addlog('[DDEpg] sr.dwSize=' + inttostr(sr.dwSize));
    addlog('[DDEpg] sr.dwStartTime=' + inttostr(sr.dwStartTime));
    addlog('[DDEpg] sr.dwStopTime=' + inttostr(sr.dwStopTime));
    addlog('[DDEpg] sr.dwChID=' + inttostr(sr.dwChID));
    }
//    Application.MessageBox(pchar(Translate('EPG_MSG1')), 'DDEpg', MB_OK or MB_ICONINFORMATION);
  fmain.Caption := 'DDEPG  - '+ Translate('EPG_MSG1');
  end;
end;

procedure Tfmain.FormShow(Sender: TObject);
var
  e: PEPGInfo;
  chinfo: TChannelInfo;
  env: TEnvironment;
begin

  env.dwSize := sizeof(TEnvironment);
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_GET_ENVIRONMENT, Integer(@env));
  BAdd.Visible := env.bSchedulerInstalled = 1;

  LV.Column[0].Caption := Translate('START_DATE');
  LV.Column[1].Caption := Translate('DURATION');
  LV.Column[2].Caption := Translate('NAME');
  BRefresh.Caption := Translate('REFRESH');
  BClear.Caption := Translate('CLEAR');
  BAdd.Caption := Translate('EPG_ADD_TO_SCHEDULER');
  GroupBox2.Caption := Translate('ELECTRONIC_PROGRAM_GUIDE');
  BRefresh.Enabled := true;
  fmain.SB.Panels[3].Text := '';

  LV.Clear;
  new(e);
  e^.dwSize := sizeof(TEPGInfo);
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_GET_EPG_CURRENT, Integer(e));
  if e.dwDuration <> 0 then
    AddToListview(e);
  new(e);
  e^.dwSize := sizeof(TEPGInfo);
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_GET_EPG_FOLLOWING, Integer(e));
  if e.dwDuration <> 0 then
    AddToListview(e);

  chinfo.dwSize := sizeof(TChannelInfo);
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_GET_CHANNEL_INFO,
    Integer(@chinfo));
  CurrentSat := chinfo.wSatPos;
  CurrentNid := chinfo.wNID;
  CurrentSid := chinfo.wSID;
  GroupBox2.Caption := Translate('ELECTRONIC_PROGRAM_GUIDE') + ' - [' + string(pchar(@chinfo.ChName)) + ']';
end;

procedure Tfmain.FormDestroy(Sender: TObject);
begin
  if EPGChs <> nil then  
    FreeMem(EPGChs);
end;

procedure Tfmain.Switchtochannel1Click(Sender: TObject);
{
var
  sch_sat: integer;
  sch_nid: integer;
  sch_sid: integer;
  sch_chid: integer;
  }
begin
{  sch_sat := EPGChs[integer(TV.Selected.data)].Sat;
  sch_nid := EPGChs[integer(TV.Selected.data)].NID;
  sch_sid := EPGChs[integer(TV.Selected.data)].SID;
  q1 := TAdoQuery.Create(fmain);
  q1.Connection := AdoCon;
  q1.SQL.Add('select * from CHANNELS where NID = ' + inttostr(sch_nid) + ' and SID=' + inttostr(sch_sid));
  q1.Open;
  if q1.RecordCount > 0 then begin
    sch_chid := q1['CHID'];
    SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_SET_CHANNEL_BY_CHID, sch_chid);
  end;
  q1.Close;
  q1.Free;}
end;

procedure Tfmain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_STOP_EPG, 0);
end;

procedure Tfmain.BClearClick(Sender: TObject);
var
  a, b: Integer;
  p: TTVChangedEvent;
begin
  p := fmain.TV.OnChange;
  fmain.TV.OnChange := nil;
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_STOP_EPG, Integer(@EPGCallback));

  application.ProcessMessages;
  sleep(100);
  application.ProcessMessages;
  sleep(100);

  for a := 0 to MAX_EPG_CHANNELS - 1 do begin
    EPGChs[a].bNotInChbase := false;
    EPGChs[a].EventsCount := 0;
    for b := 0 to EPGChs[a].EventsCount - 1 do
      Dispose(EPGChs[a].Events[b]);
  end;
  FreeMem(EPGChs);
  EPGChsCount := 0;
  fmain.LV.Items.Clear;
  fmain.TV.Items.Clear;
  Reset;
  fmain.SB.Panels.Items[0].Text := IntToStr(EPGChsCount) + ' ' + Translate('EPG_CHANNELS_SHORT');
  fmain.SB.Panels.Items[1].Text := inttostr(TotalEPG) + ' ' + Translate('EPG_EVENTS');

  fmain.TV.OnChange := p;
  SendMessage(hwndDD, WM_MODULE_MSG, DDMODAPI_START_EPG, Integer(@EPGCallback));
end;

procedure Tfmain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case key of
    VK_ESCAPE: fmain.Close;
    else
      key := 0;  // ignore all other keys
  end;
end;

end.

